#
# Pythagoreische Zahlentripel
#
# Eingabe: eine ganze Zahl c > 0
#
# Ausgabe: wenn es sie gibt, zwei ganze Zahlen
#   a >0 und b > 0, so dass a^2+b^2 = c^2,
#   sonst eine Meldung, dass es keine Loesung gibt
#
#######################################################

import sys

# Einzulesende Zahl aus den Argumenten holen
try:
    c = int(sys.argv[1])
except:
    sys.stderr.write("Bitte eine ganze Zahl eingeben (%s ZAHL)\n" % sys.argv[0])
    quit()

def zahlentripel_n2(c):
    """
    Liefert ein Ganzzahl-Paar (a, b) mit a > 0 und b > 0, so dass
    a^2 + b^2 = c^2, oder None, wenn keine solche Loesung existiert.
    """

    # Probiert einfach die Zahlen a und b durch, um
    # eine Loesung zu finden. Klar ist: 0 < a < c und
    # 0 < b < c, ansonsten werden alle Paare durch probiert

    for a in range(1,c):
        for b in range(1,c):
            if a**2 + b**2 == c**2: return (a, b)
    return None

def zahlentripel_n(c):
    """
    Liefert ein Ganzzahl-Paar (a, b) mit a > 0 und b > 0, so dass
    a^2 + b^2 = c^2, oder None, wenn keine solche Loesung existiert.
    """

    # Intelligenteres Verfahren
    # Klar ist wieder : 0 < a < c und 0 < b < c
    # Seien zunaechst a=1 und b=c-1. Ist nun a^2 + b^2 < c^2,
    # so muss a vergroessert werden, da b ja schon maximal ist,
    # und es gibt keine Loesung mit a = 1
    # Umgekehrt muss b verkleinert werden, falls a^2 + b^2 > c^2.
    # In diesem Fall gibt es keine Loesung mit b = c - 1
    # Dies koennen wir nun fortfuehren, bis wir entweder eine Loesung
    # finden, oder a > b ist. Dann gibt es keine Loesung
    
    a = 1
    b = c - 1
    while a <= b:
        if   a**2 + b**2 < c**2: a += 1
        elif a**2 + b**2 > c**2: b -= 1
        else: return (a, b)
    return None

res = zahlentripel_n(c)

if res != None:
    a, b = res
    print "Loesung gefunden: %d^2 + %d^2 = %d^2" % (a, b, c)
else:
    print "Es gibt keine positive ganzzahlige Loesung von a^2 + b^2 = %d^2" % c
