#
# Berechnet rekursive die Determinate einer quadratischen Matrix durch
# Entwicklung nach der ersten Zeile.
#
# Gegentest mit der eingebauten Funktion von numpy
#
##########################################################

import numpy as np

def determinante(a):
    "berechnet die Determinante von a, wenn a quadratisch ist"

    # Form ueberpruefen
    ###########################
    n, m = a.shape
    if n != m:
        raise Exception("Matrix ist nicht quadratisch")

    # Ende der Rekursion, bei Skalaren ist die Determinante
    # einfach der Skalar selber
    ###########################
    if n == 1:
        return a[0,0]

    # Entwicklung nach Spalte 0
    ###########################

    # laufende Summe
    det = 0
    # Signum der Permutation
    faktor = 1
    # Ablaufen der Zeilen
    for i in range(n):
        # Matrix a mit Spalte 0 und Zeile i gestrichen
        submatrix = a[range(0, i) + range(i+1, n),1:]
        
        det += faktor*a[i,0]*determinante(submatrix)

        faktor *= -1

    return det

# Demonstration mit einer singulaeren Matrix
a = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
print determinante(a), np.linalg.det(a)

# und mit einer Diagonalmatrix
a = np.array([[1, 0, 0], [0, 2, 0], [0, 0, 3]])
print determinante(a), np.linalg.det(a)
