#
# Demonstration der linearen Algebra in numpy
#
# Gegeben: eine Funktion f als diskrete Werte an den Stellen
# 0, h, 2h, 4h,...
#
# Gesucht: eine Funktion g, definiert an den selben Stuetzstellen, so
# dass g'' = f, d. h. wir integrieren f zweifach
#
# Methode (nicht optimal, aber moeglich):
# Wir diskretisieren die Ableitung als
#      f'(x) = (f(x) - f(x - h))/h
# und
#      f''(x) = (f'(x + h) - f'(x))/h =
#           ( f(x-h) - 2 f(x) + f(x + h)) / h**2
#
# Ist nun f[i] = f(x + i*h), so ist f''[i] = f[i-1] - 2*f[i] + f[i+1],
# was einer Matrix D mit einer -2 auf der Diagonalen und 1 auf den
# Nebendiagonalen entspricht. Wir loesen nun Dg = f
#
# Bemerkung: Am Rand fehlen natuerlich Werte ausserhalb des Bereichs.
# Wir setzen durch die Form von D diese Elemente auf Null, so dass g
# an beiden Raendern stets 0 ist.
#
##################################################

import numpy as np

# Gitterweite
h = 0.01

# Unsere Funktion f
#f = np.sin(np.arange(0, 2*np.pi, h))
f = np.arange(0, 1, h)

n = f.shape[0]

# Matrix mit 1/h**2 , -2/h**2, 1/h**2 auf der Diagonalen,
# entsprechend ( f(x-h) - 2 f(x) + f(x + h)) / h**2

# Einheitsmatrix durch h**2
I = np.diag(np.ones(n)) / h**2
D = -2*I

# Nebendiagonalen durch Verschieben der Einheitsmatrix nach oben
# und Anhaengen einer Leerzeile
D += np.concatenate((I[1:,:], np.zeros(n).reshape((1,n))))
D += np.concatenate((np.zeros(n).reshape((1,n)), I[:n-1,:]))

# Loesen der Gleichung

g = np.linalg.solve(D, f)

# Alternativ per Inverser, was aber langsamer ist

galt = np.dot(np.linalg.inv(D), f)

x = 0
for vf, vg, vgalt in zip(f, g, galt):
    print x, vf, vg, vgalt
    x += h

